<?php

App::uses('AppBehavior', 'Model/Behavior');
App::uses('ArrayUtil', 'EvCore.Lib');

class RelatableBehavior extends AppBehavior {

	/**
	 * bind the related model to the model
	 *
	 */
	public function linkRelatedItem(&$Model) {
		$Model->bindModel(
			array(
				'hasMany' => array(
					'RelatedItem' => array(
						'className' => 'EvRelatedItems.RelatedItem',
						'foreignKey' => 'model_id',
						'conditions' => array(
							'RelatedItem.model' => EvClassRegistry::getNameFromModel($Model)
						)
					)
				)
			),
			false
		);
	}

	/**
	 * process any options and validation into json for storage
	 *
	 * @param Model $model Model using this behavior
	 * @param array $options Options passed from Model::save().
	 * @return mixed False if the operation should abort. Any other result will continue.
	 * @see Model::save()
	 */
	public function beforeBeforeSave(&$Model, $data, $options = array()) {
		$this->linkRelatedItem($Model);

		if (! empty($data['RelatedItem'])) {
			$modelAlias = EvClassRegistry::getNameFromModel($Model);

			foreach ($data['RelatedItem'] as $key => $discount) {
				$data['RelatedItem'][$key]['model'] = $modelAlias;
			}
		}

		return $data;
	}

	/**
	 * Save the discounts
	 *
	 * @param Model $model Model using this behavior
	 * @param bool $created True if this save created a new record
	 * @param array $options Options passed from Model::save().
	 * @return bool
	 * @see Model::save()
	 */
	public function afterSave(Model $Model, $created, $options = array()) {
		$return = parent::afterSave($Model, $created, $options);
		$result = true;

		$RelatedItem = EvClassRegistry::init('EvRelatedItems.RelatedItem');

		// process any custom field deletions
		if (! empty($Model->data[$Model->alias]['RelatedItems']['toDelete'])) {

			$ids = explode(',', $Model->data[$Model->alias]['RelatedItems']['toDelete']);
			$ids = ArrayUtil::clearArray($ids);

			$RelatedItem->deleteAll(
				array(
					'RelatedItem.id' => $ids
				),
				true,
				true
			);
		}

		return $return;
	}

	/**
	 * beforeFind can be used to cancel find operations, or modify the query that will be executed.
	 * By returning null/false you can abort a find. By returning an array you can modify/replace the query
	 * that is going to be run.
	 *
	 * @param Model $model Model using this behavior
	 * @param array $query Data used to execute this query, i.e. conditions, order, etc.
	 * @return bool|array False or null will abort the operation. You can return an array to replace the
	 *   $query that will be eventually run.
	 */
	public function beforeFind(Model $Model, $query) {
		$this->linkRelatedItem($Model);

		$query['contain'][] = 'RelatedItem';

		return $query;
	}

	/**
	 * After find callback. Can be used to modify any results returned by find.
	 *
	 * @param Model $model Model using this behavior
	 * @param mixed $results The results of the find operation
	 * @param bool $primary Whether this model is being queried directly (vs. being queried as an association)
	 * @return mixed An array value will replace the value of $results - any other value will be ignored.
	 */
	public function afterFind(Model $model, $results, $primary = false) {
		// Perform after find on any related items to get the "displayField / id" for the related item
		// then in template check for a specific array value to prepopulate options on related_model_id
	}
}
